#!/usr/bin/env python3

import os
import sys

name = sys.argv[1]
value = []
root = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
with open(os.path.join(root, 'src', 'Iaito.pro')) as f:
    text = f.read()
text = text.replace('\\\n', '')
nbraces = 0
for line in text.split('\n'):
    line = line.strip()
    if not line or all(char in line for char in ('{', '}')):
        continue
    if line.startswith('}'):
        nbraces -= 1
        continue
    if line.endswith('{'):
        nbraces += 1
        continue
    if nbraces > 0 or '=' not in line:
        continue
    words = line.split()
    if words[0] == name and '=' in words[1]:
        value.extend(words[2:])
if name == 'QT':
    value = [str.title(s) for s in value]
if not value:
    sys.exit(1)
print(';'.join(value), end='')



#!/usr/bin/env python3
import os
import re

scripts_path = os.path.dirname(os.path.realpath(__file__))
pro_file_path = os.path.join(scripts_path, "..", "src", "Iaito.pro")

with open(pro_file_path, "r") as f:
    pro_content = f.read()

def version_var_re(name):
    return "^[ \t]*{}[ \t]*=[ \t]*(\d+)[ \t]*$".format(name)

m = re.search(version_var_re("IAITO_VERSION_MAJOR"), pro_content, flags=re.MULTILINE)
version_major = int(m.group(1)) if m is not None else 0

m = re.search(version_var_re("IAITO_VERSION_MINOR"), pro_content, flags=re.MULTILINE)
version_minor = int(m.group(1)) if m is not None else 0

m = re.search(version_var_re("IAITO_VERSION_PATCH"), pro_content, flags=re.MULTILINE)
version_patch = int(m.group(1)) if m is not None else 0

print("{}.{}.{}".format(version_major, version_minor, version_patch))




import iaito

from PySide2.QtCore import Qt
from PySide2.QtWidgets import QVBoxLayout, QLabel, QWidget, QSizePolicy, QPushButton


class FortuneWidget(iaito.IaitoDockWidget):
    def __init__(self, parent):
        super(FortuneWidget, self).__init__(parent)
        self.setObjectName("FancyDockWidgetFromCoolPlugin")
        self.setWindowTitle("Sample Python Plugin")

        content = QWidget()
        self.setWidget(content)

        # Create layout and label
        layout = QVBoxLayout(content)
        content.setLayout(layout)
        self.text = QLabel(content)
        self.text.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        self.text.setFont(iaito.Configuration.instance().getFont())
        layout.addWidget(self.text)

        button = QPushButton(content)
        button.setText("Want a fortune?")
        button.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)
        button.setMaximumHeight(50)
        button.setMaximumWidth(200)
        layout.addWidget(button)
        layout.setAlignment(button, Qt.AlignHCenter)

        button.clicked.connect(self.generate_fortune)
        iaito.core().seekChanged.connect(self.generate_fortune)

        self.show()

    def generate_fortune(self):
        fortune = iaito.cmd("fo").replace("\n", "")
        res = iaito.core().cmdRaw(f"?E {fortune}")
        self.text.setText(res)


class IaitoSamplePlugin(iaito.IaitoPlugin):
    name = "Sample Plugin"
    description = "A sample plugin written in python."
    version = "1.1"
    author = "Iaito developers"

    # Override IaitoPlugin methods

    def __init__(self):
        super(IaitoSamplePlugin, self).__init__()
        self.disassembly_actions = []
        self.addressable_item_actions = []
        self.disas_action = None
        self.addr_submenu = None
        self.main = None

    def setupPlugin(self):
        pass

    def setupInterface(self, main):
        # Dock widget
        widget = FortuneWidget(main)
        main.addPluginDockWidget(widget)

        # Dissassembly context menu
        menu = main.getContextMenuExtensions(iaito.MainWindow.ContextMenuType.Disassembly)
        self.disas_action = menu.addAction("IaitoSamplePlugin dissassembly action")
        self.disas_action.triggered.connect(self.handle_disassembler_action)
        self.main = main

        # Context menu for tables with addressable items like Flags,Functions,Strings,Search results,...
        addressable_item_menu = main.getContextMenuExtensions(iaito.MainWindow.ContextMenuType.Addressable)
        self.addr_submenu = addressable_item_menu.addMenu("IaitoSamplePlugin") # create submenu
        adrr_action = self.addr_submenu.addAction("Action 1")
        self.addr_submenu.addSeparator() # can use separator and other qt functionality
        adrr_action2 = self.addr_submenu.addAction("Action 2")
        adrr_action.triggered.connect(self.handle_addressable_item_action)
        adrr_action2.triggered.connect(self.handle_addressable_item_action)

    def terminate(self): # optional
        print("IaitoSamplePlugin shutting down")
        if self.main:
            menu = self.main.getContextMenuExtensions(iaito.MainWindow.ContextMenuType.Disassembly)
            menu.removeAction(self.disas_action)
            addressable_item_menu = self.main.getContextMenuExtensions(iaito.MainWindow.ContextMenuType.Addressable)
            submenu_action = self.addr_submenu.menuAction()
            addressable_item_menu.removeAction(submenu_action)
        print("IaitoSamplePlugin finished clean up")

    # Plugin methods

    def handle_addressable_item_action(self):
        # for actions in plugin menu Iaito sets data to current item address
        submenu_action = self.addr_submenu.menuAction()
        iaito.message("Context menu action callback 0x{:x}".format(submenu_action.data()))

    def handle_disassembler_action(self):
        # for actions in plugin menu Iaito sets data to address for current dissasembly line
        iaito.message("Dissasembly menu action callback 0x{:x}".format(self.disas_action.data()))


# This function will be called by Iaito and should return an instance of the plugin.
def create_iaito_plugin():
    return IaitoSamplePlugin()



#!/bin/env python3

import os
import xml.etree.ElementTree as et
import sys


script_path = os.path.dirname(os.path.realpath(__file__))


def get_cpp_files_gen(args, include_package=True):
    ts_tree = et.parse(os.path.join(script_path, "bindings.xml"))
    ts_root = ts_tree.getroot()

    package = ts_root.attrib["package"]

    type_tags = ["object-type", "value-type"]
    types = [child.attrib["name"] for child in ts_root if child.tag in type_tags]

    cpp_files_gen = [f"{package.lower()}_module_wrapper.cpp"]
    cpp_files_gen.extend([f"{typename.lower()}_wrapper.cpp" for typename in types])

    if include_package:
        cpp_files_gen = [os.path.join(package, f) for f in cpp_files_gen]

    if len(args) > 0:
        cpp_files_gen = [os.path.join(args[0], f) for f in cpp_files_gen]

    return cpp_files_gen


def cmd_cmake(args):
    files = get_cpp_files_gen(args)
    if sys.platform == "win32":
        files = map(lambda x: x.replace("\\", "/"), files)
    sys.stdout.write(";".join(files))


def cmd_qmake(args):
    sys.stdout.write("\n".join(get_cpp_files_gen(args)) + "\n")


def cmd_meson(args):
    sys.stdout.write(";".join(get_cpp_files_gen(args, include_package=False)))


cmds = {"cmake": cmd_cmake, "qmake": cmd_qmake, "meson": cmd_meson}

if len(sys.argv) < 2 or sys.argv[1] not in cmds:
    print(f"""usage: {sys.argv[0]} [{"/".join(cmds.keys())}] [base path]""")
    exit(1)
cmds[sys.argv[1]](sys.argv[2:])



#!/usr/bin/env python3

import sys
import re
import json

in_filename = sys.argv[1]
out_filename = sys.argv[2]
vars = json.loads(sys.argv[3])

with open(in_filename, "r") as f:
    content = f.read()

content = content.replace("\\\"", "\"")

varname_pattern = re.compile("[A-Za-z0-9_]+")
for name, value in vars.items():
    if varname_pattern.fullmatch(name) is None:
        print("Name \"{}\" is not a valid variable name.".format(name))
        continue

    pattern = "\\$\\$({}|\\{{{}\\}})".format(name, name)
    print(pattern)
    content = re.sub(pattern, str(value), content)

with open(out_filename, "w") as f:
    f.write(content)



#!/usr/bin/env python
import xml.etree.ElementTree as ET
import subprocess
import sys

def remove_lines_with_dashes(input_string):
    lines = input_string.splitlines()
    filtered_lines = [line for line in lines if "----------" not in line]
    return "\n".join(filtered_lines).strip()

# Define the translation function
def translate_text(input_text, existing_translation, language):
    """
    Calls an external program to translate the input text, using existing translation if provided.

    Args:
        input_text (str): The text to translate.
        existing_translation (str): The existing translation to provide context.

    Returns:
        str: The translated text.
    """
    inspiration = ""
    if existing_translation and existing_translation != "":
        inspiration = ". Take this original text as inspiration: '"+existing_translation+"'"
    try:
        # Replace 'translation_program' with the actual program/command
        process = subprocess.Popen(
            ['/usr/local/bin/ai', '--', '"translate this sentence to '+language+', the output must be just the text, the context for this translation is the Iaito program, which is the graphical interface for radare2, a reverse engineering tool for analyzing and debugging programs'+inspiration+'"'],
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            text=True
        )
        print('"translate this sentence to '+language+', the output must be just the text, the context for this translation is the Iaito program, which is the graphical interface for radare2, a reverse engineering tool for analyzing and debugging programs'+inspiration+'"')
        input_data = f"{input_text}"
        stdout, stderr = process.communicate(input=input_data)

        if process.returncode != 0:
            raise RuntimeError(f"Translation program error: {stderr.strip()}")

        print(stdout)
        return remove_lines_with_dashes(stdout)
    except Exception as e:
        print(f"Error during translation: {e}", file=sys.stderr)
        return input_text  # Fallback to the original text if an error occurs

# Define the function to find existing translation
def find_existing_translation(existing_tree, source_text):
    """
    Finds the translation for a given source text in an existing .ts file.

    Args:
        existing_tree (ElementTree): Parsed XML tree of the existing .ts file.
        source_text (str): The source text to look for.

    Returns:
        str: The existing translation if found, else an empty string.
    """
    for context in existing_tree.findall('context'):
        for message in context.findall('message'):
            source = message.find('source')
            translation = message.find('translation')
            if source is not None and source.text == source_text and translation is not None:
                return translation.text
    return ""

# Define the main function to parse and modify the XML file
def process_ts_file(input_file, output_file, existing_file, language):
    """
    Parses the .ts file, adds a <translation> node with translated content, and writes the output.

    Args:
        input_file (str): Path to the input .ts file.
        output_file (str): Path to save the modified .ts file.
        existing_file (str): Path to an existing .ts file to find previous translations.
    """
    tree = ET.parse(input_file)
    root = tree.getroot()

    existing_tree = ET.parse(existing_file)

    for context in root.findall('context'):
        for message in context.findall('message'):
            source = message.find('source')
            if source is not None and source.text:
                original_text = source.text
                existing_translation = find_existing_translation(existing_tree, original_text)
                translated_text = translate_text(original_text, existing_translation, language)
                # Check if <translation> node exists, if not create it
                translation = message.find('translation')
                if translation is None:
                    translation = ET.SubElement(message, 'translation')
                translation.text = translated_text
                if 'type' in translation.attrib:
                    del translation.attrib['type']

    # Save the updated XML to the output file
    tree.write(output_file, encoding='utf-8', xml_declaration=True)

if __name__ == "__main__":
    if len(sys.argv) <= 4:
        print("Usage: python script.py input.ts output.ts existing.ts (Language)", file=sys.stderr)
        sys.exit(1)

    input_ts = sys.argv[1]
    output_ts = sys.argv[2]
    existing_ts = sys.argv[3]
    if len(sys.argv) <= 5:
        language = sys.argv[4]

    process_ts_file(input_ts, output_ts, existing_ts, language)
    print(f"Translated file saved to {output_ts}")

